//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Util/UtilXML.h
//! @brief     Defines
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_UTIL_UTILXML_H
#define BORNAGAIN_GUI_MODEL_UTIL_UTILXML_H

#include <QColor>
#include <QXmlStreamReader> // used in every including file, also provides QXmlStreamWriter

namespace XML {

constexpr const char* LinkMimeType{"application/org.bornagainproject.fittinglink"};

namespace Tag {

const QString BaseData("BaseData");

} // namespace Tag

namespace Attrib {

const QString BA_Version("BA_Version");
const QString collapsed("collapsed");
const QString id("id");
const QString index("index");
const QString name("name");
const QString projectName("projectName");
const QString selection_version("selection_version");
const QString type("type");
const QString value("value");
const QString version("version");
const QString xPos("xPos");
const QString yPos("yPos");

} // namespace Attrib

void gotoEndElementOfTag(QXmlStreamReader* reader, const QString& tag);

void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName, bool b);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName, double d);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName, QString s);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName, QColor c);

template <typename T>
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName, const T d)
{
    writer->writeAttribute(attributeName, QString::number(d));
};

template <typename T> void writeTaggedValue(QXmlStreamWriter* w, const QString& tag, const T value)
{
    w->writeStartElement(tag);
    writeAttribute(w, XML::Attrib::value, value);
    w->writeEndElement();
};

template <typename T>
void writeTaggedElement(QXmlStreamWriter* w, const QString& tag, const T& element)
{
    w->writeStartElement(tag);
    element.writeTo(w);
    w->writeEndElement();
}

template <typename T> void writeBaseElement(QXmlStreamWriter* w, const T* element)
{
    w->writeStartElement(Tag::BaseData);
    element->T::writeTo(w);
    w->writeEndElement();
}

bool readBool(QXmlStreamReader* reader, const QString& attributeName);
int readInt(QXmlStreamReader* reader, const QString& attributeName);
unsigned readUInt(QXmlStreamReader* reader, const QString& attributeName);
double readDouble(QXmlStreamReader* reader, const QString& attributeName);
QString readString(QXmlStreamReader* reader, const QString& attributeName);

bool readTaggedBool(QXmlStreamReader* reader, const QString& tag);
int readTaggedInt(QXmlStreamReader* reader, const QString& tag);
unsigned readTaggedUInt(QXmlStreamReader* reader, const QString& tag);
size_t readTaggedSize(QXmlStreamReader* reader, const QString& tag);
double readTaggedDouble(QXmlStreamReader* reader, const QString& tag);
QString readTaggedString(QXmlStreamReader* reader, const QString& tag);
QColor readTaggedColor(QXmlStreamReader* reader, const QString& tag);

template <typename T> void readTaggedElement(QXmlStreamReader* r, const QString& tag, T& element)
{
    element.readFrom(r);
    gotoEndElementOfTag(r, tag);
}

template <typename T> void readBaseElement(QXmlStreamReader* r, T* element)
{
    element->T::readFrom(r);
    gotoEndElementOfTag(r, Tag::BaseData);
}

} // namespace XML

#endif // BORNAGAIN_GUI_MODEL_UTIL_UTILXML_H
